/*
Copyright (c) 1992-2007 The University of Tennessee.  All rights reserved.

$COPYRIGHT$

Additional copyrights may follow

$HEADER$

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
 met:

- Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer. 
  
- Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer listed
  in this license in the documentation and/or other materials
  provided with the distribution.
  
- Neither the name of the copyright holders nor the names of its
  contributors may be used to endorse or promote products derived from
  this software without specific prior written permission.
  
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT  
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT  
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
*/
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <complex>

#include <mblas.h>
#include "blas.h"

#define MAX_INCX  10
#define MIN_INCX -10
#define MAX_INCY  10
#define MIN_INCY -10
#define MAX_N     10
#define MAX_ITER  10

#include "mblas.debug.h"

void
Caxpy_test2(mpc_class alpha)
{
    int errorflag = FALSE;
    mpc_class ctemp;
    double dtemp;

    complex < double >alphad;

    alphad.real() = alpha.re.get_d();
    alphad.imag() = alpha.im.get_d();

    for (int incx = MAX_INCX; incx >= MIN_INCX; incx--) {
	for (int incy = MAX_INCY; incy >= MIN_INCY; incy--) {
	    for (int n = 0; n < MAX_N; n++) {
		printf("# n:%d incx:%d, incy:%d\n", n, incx, incy);

		mpc_class *x = new mpc_class[VEC_A(n, incx)];
		mpc_class *y = new mpc_class[VEC_A(n, incy)];
		complex < double >*xd = new complex < double >[VEC_A(n, incx)];
		complex < double >*yd = new complex < double >[VEC_A(n, incy)];
		int j = 0;

		while (j < MAX_ITER) {
		    set_random_vector(x, xd, VEC_A(n, incx));
		    set_random_vector(y, yd, VEC_A(n, incy));

		    Caxpy(n, alpha, x, incx, y, incy);
		    zaxpy_f77(&n, &alphad, xd, &incx, yd, &incy);

		    double diff = vec_diff(y, yd, VEC_A(n, incy), 1);

		    if (fabs(diff) > EPSILON) {
			printf("error %lf!!\n", diff);
			errorflag = TRUE;
		    }
		    j++;
		}
		delete[]x;
		delete[]xd;
		delete[]y;
		delete[]yd;
	    }
	}
    }
    if (errorflag == TRUE) {
	printf("Caxpy test failed...\n");
    } else {
	printf("Caxpy test passed...\n");
    }
}

void
Caxpy_test()
{
    mpc_class alpha;

    alpha.re = 0.0;
    alpha.im = 0.0;
    Caxpy_test2(alpha);

    alpha = mpc_randomnumber();
    Caxpy_test2(alpha);
}

int
main(int argc, char *argv[])
{
//initialization
    int default_prec = 200;
    gmp_randstate_t state;

    mpf_set_default_prec(default_prec);
    uniformrandom = new gmp_randclass(gmp_randinit_default);
    VERBOSE = 0;
    Caxpy_test();
}
